@@ -1,4 +1,5 @@
 
+#include <stdio.h>
 #include "pvm3.h"
 
 main()
@@ -20,6 +21,7 @@ main()
 		printf("can't start hello_other\n");
 
 	pvm_exit();
+	exit(0);
 }
 
 
