@@ -1,6 +1,6 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -32,6 +32,27 @@
  *	Host table functions.
  *
 $Log: host.c,v $
+ * Revision 1.10  1996/05/13  20:25:48  manchek
+ * missing pvmlogerror in readhostfile
+ *
+ * Revision 1.9  1995/11/02  16:01:49  manchek
+ * use NET_IF_IN_SYS if include/sys/net/if.h
+ *
+ * Revision 1.8  1995/07/28  16:51:30  manchek
+ * ifdef for UTS2
+ *
+ * Revision 1.7  1995/05/17  16:09:13  manchek
+ * added HF_OVERLOAD and '$' syntax
+ *
+ * Revision 1.6  1994/10/15  19:05:39  manchek
+ * must use PVM_FREE instead of free in iflist()
+ *
+ * Revision 1.5  1994/06/21  18:30:55  manchek
+ * HF_SPEED flag now determines whether hd_speed has been set
+ *
+ * Revision 1.4  1994/06/03  20:38:14  manchek
+ * version 3.3.0
+ *
  * Revision 1.3  1993/11/30  19:52:05  manchek
  * readhostfile() puts defaults in ht_hosts[0]
  *
@@ -54,7 +75,7 @@ $Log: host.c,v $
 #include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#ifdef	IMA_E88K
+#ifdef	NET_IF_IN_SYS
 #include <sys/net/if.h>
 #else
 #include <net/if.h>
@@ -105,7 +126,7 @@ extern int tidhmask;				/* from pvmd.c */
  **           **
  ***************/
 
-static char rcsid[] = "$Id: host.c,v 1.3 1993/11/30 19:52:05 manchek Exp $";
+static char rcsid[] = "$Id: host.c,v 1.10 1996/05/13 20:25:48 manchek Exp $";
 static char pvmtxt[1024];			/* scratch space for error messages */
 
 
@@ -167,6 +188,10 @@ hd_free(hp)
 		PVM_FREE(hp->hd_epath);
 	if (hp->hd_bpath)
 		PVM_FREE(hp->hd_bpath);
+	if (hp->hd_wdir)
+		PVM_FREE(hp->hd_wdir);
+	if (hp->hd_sopts)
+		PVM_FREE(hp->hd_sopts);
 	if (hp->hd_txq)
 		pk_free(hp->hd_txq);
 	if (hp->hd_opq)
@@ -174,12 +199,14 @@ hd_free(hp)
 	if (hp->hd_rxq)
 		pk_free(hp->hd_rxq);
 	if (hp->hd_rxm)
-		mesg_free(hp->hd_rxm);
+		mesg_unref(hp->hd_rxm);
 	if (mcap = hp->hd_mcas) {
 		while (mcap->mc_link != mcap)
 			mca_free(mcap->mc_link);
 		mca_free(mcap);
 	}
+	if (hp->hd_aname)
+		PVM_FREE(hp->hd_aname);
 	PVM_FREE(hp);
 }
 
@@ -205,12 +232,24 @@ hd_dump(hp)
 	int n;
 	struct pkt *pp, *pp2;
 
-	sprintf(pvmtxt, " hd_dump() ref %d t%x n \"%s\" ar \"%s\" lo \"%s\"\n",
+	sprintf(pvmtxt,
+			" hd_dump() ref %d t%x n \"%s\" a \"%s\" ar \"%s\"\n",
 			hp->hd_ref,
 			hp->hd_hostpart,
 			(hp->hd_name ? hp->hd_name : ""),
-			(hp->hd_arch ? hp->hd_arch : ""),
-			(hp->hd_login ? hp->hd_login : ""));
+			(hp->hd_aname ? hp->hd_aname : ""),
+			(hp->hd_arch ? hp->hd_arch : ""));
+	pvmlogerror(pvmtxt);
+
+	sprintf(pvmtxt,
+			"           lo \"%s\" so \"%s\" dx \"%s\" ep \"%s\" bx \"%s\" wd \"%s\" sp %d\n",
+			(hp->hd_login ? hp->hd_login : ""),
+			(hp->hd_sopts ? hp->hd_sopts : ""),
+			(hp->hd_dpath ? hp->hd_dpath : ""),
+			(hp->hd_epath ? hp->hd_epath : ""),
+			(hp->hd_bpath ? hp->hd_bpath : ""),
+			(hp->hd_wdir ? hp->hd_wdir : ""),
+			hp->hd_speed);
 	pvmlogerror(pvmtxt);
 
 	n = 0;
@@ -387,7 +426,7 @@ ht_insert(htp, hp)
 	for (hh = htp->ht_last; hh > 0; hh--) {
 		hp = htp->ht_hosts[hh];
 		if (hp && hp->hd_arch) {
-			i = getarchcode(htp->ht_hosts[hh]->hd_arch);
+			i = pvm_archcode(htp->ht_hosts[hh]->hd_arch);
 			tmpmask = (1 << i);
 			if (tmpmask & ~mask)
 				htp->ht_narch++;
@@ -428,7 +467,7 @@ ht_delete(htp, hp)
 		if (!(hp = htp->ht_hosts[hh]))
 			continue;
 		if (hp->hd_arch) {
-			i = getarchcode(htp->ht_hosts[hh]->hd_arch);
+			i = pvm_archcode(htp->ht_hosts[hh]->hd_arch);
 			tmpmask = (1 << i);
 			if (tmpmask & ~mask)
 				htp->ht_narch++;
@@ -473,6 +512,136 @@ ht_merge(dst, src)
 }
 
 
+/*	applydefaults()
+*
+*	Apply default settings to unspecified fields of hostd.
+*/
+
+int
+applydefaults(hp, defhp)
+	struct hostd *hp;		/* hostd to modify */
+	struct hostd *defhp;	/* default settings */
+{
+	if (!hp->hd_login && defhp->hd_login)
+		hp->hd_login = STRALLOC(defhp->hd_login);
+
+	if (!hp->hd_dpath && defhp->hd_dpath)
+		hp->hd_dpath = STRALLOC(defhp->hd_dpath);
+
+	if (!hp->hd_epath && defhp->hd_epath)
+		hp->hd_epath = STRALLOC(defhp->hd_epath);
+
+	if (!hp->hd_bpath && defhp->hd_bpath)
+		hp->hd_bpath = STRALLOC(defhp->hd_bpath);
+
+	if (!hp->hd_wdir && defhp->hd_wdir)
+		hp->hd_wdir = STRALLOC(defhp->hd_wdir);
+
+	if (!hp->hd_sopts && defhp->hd_sopts)
+		hp->hd_sopts = STRALLOC(defhp->hd_sopts);
+
+	if (!hp->hd_aname && defhp->hd_aname)
+		hp->hd_aname = STRALLOC(defhp->hd_aname);
+
+	if (!(hp->hd_flag & HF_SPEED) && (defhp->hd_flag & HF_SPEED))
+		hp->hd_speed = defhp->hd_speed;
+
+	hp->hd_flag |= defhp->hd_flag;
+
+	return 0;
+}
+
+
+/*	parsehost()
+*
+*	Parse hostfile line into hostd.
+*/
+
+int
+parsehost(buf, hp)
+	char *buf;
+	struct hostd *hp;
+{
+	char *av[10];		/* parsed words */
+	int ac;
+	int err = 0;
+
+	ac = sizeof(av)/sizeof(av[0]);
+	if (acav(buf, &ac, av)) {
+		sprintf(pvmtxt, "parsehost(): line too long\n");
+		pvmlogerror(pvmtxt);
+		goto bad;
+	}
+	if (!ac)
+		goto bad;
+
+	/* add options to host descriptor */
+
+	while (--ac > 0) {
+		if (!strncmp(av[ac], "lo=", 3)) {
+			if (hp->hd_login)
+				PVM_FREE(hp->hd_login);
+			hp->hd_login = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		if (!strncmp(av[ac], "dx=", 3)) {
+			if (hp->hd_dpath)
+				PVM_FREE(hp->hd_dpath);
+			hp->hd_dpath = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		if (!strncmp(av[ac], "ep=", 3)) {
+			if (hp->hd_epath)
+				PVM_FREE(hp->hd_epath);
+			hp->hd_epath = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		if (!strncmp(av[ac], "sp=", 3)) {
+			hp->hd_speed = atoi(av[ac] + 3);
+			hp->hd_flag |= HF_SPEED;
+			continue;
+		}
+		if (!strncmp(av[ac], "bx=", 3)) {
+			if (hp->hd_bpath)
+				PVM_FREE(hp->hd_bpath);
+			hp->hd_bpath = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		if (!strncmp(av[ac], "wd=", 3)) {
+			if (hp->hd_wdir)
+				PVM_FREE(hp->hd_wdir);
+			hp->hd_wdir = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		if (!strncmp(av[ac], "so=", 3)) {
+			if (hp->hd_sopts)
+				PVM_FREE(hp->hd_sopts);
+			hp->hd_sopts = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		if (!strncmp(av[ac], "ip=", 3)) {
+			if (hp->hd_aname)
+				PVM_FREE(hp->hd_aname);
+			hp->hd_aname = STRALLOC(av[ac] + 3);
+			continue;
+		}
+		sprintf(pvmtxt, "parsehost(): unknown option \"%s\"\n", av[ac]);
+		pvmlogerror(pvmtxt);
+		err++;
+	}
+	if (err)
+		goto bad;
+
+	if (hp->hd_name)
+		PVM_FREE(hp->hd_name);
+	hp->hd_name = STRALLOC(av[0]);
+	return 0;
+
+bad:
+	return -1;
+}
+
+
 /*	readhostfile()
 *
 *	Read a host file and return host table with options and ipaddrs
@@ -488,16 +657,14 @@ readhostfile(fn)
 	FILE *ff = 0;
 	char buf[512];		/* line buffer */
 	int lnum = 0;		/* line counter */
-	char *av[10];		/* parsed words of hostfile line */
-	int ac;
 	int err = 0;		/* error count */
 	struct hostent *he;
 	char *p;
 	struct in_addr *my_in_addrs;
 	int num_addrs;
 	int i;
-	struct hostd *defaults = 0;
 	int maxhostid = tidhmask >> (ffs(tidhmask) - 1);
+	struct hostd *defaults = 0;
 
 	htp = ht_new(1);
 
@@ -507,6 +674,7 @@ readhostfile(fn)
 
 	if (iflist(&my_in_addrs, &num_addrs) == -1 || num_addrs < 1) {
 		sprintf(pvmtxt, "readhostfile() iflist failed\n");
+		pvmlogerror(pvmtxt);
 		goto bail;
 	}
 
@@ -526,85 +694,34 @@ readhostfile(fn)
 			continue;
 
 		if (lnum > maxhostid) {
-			pvmlogerror("readhostfile() %s: too many lines\n", fn);
+			sprintf(pvmtxt, "readhostfile() %s: too many lines\n", fn);
+			pvmlogerror(pvmtxt);
 			err++;
 			break;
 		}
 		hp = hd_new(lnum);
 
-		if (*p == '&') {		/* leading '&' is no-start */
-			hp->hd_flag |= HF_NOSTART;
+		while (*p == '&' || *p == '$') {
+			if (*p == '&')		/* leading '&' is no-start */
+				hp->hd_flag |= HF_NOSTART;
+			if (*p == '$')		/* leading '$' allows overloading host */
+				hp->hd_flag |= HF_OVERLOAD;
 			p++;
 		}
 
-		ac = sizeof(av)/sizeof(av[0]);
-		if (acav(p, &ac, av)) {
-			sprintf(pvmtxt, "readhostfile() %s %d: line too long\n", fn, lnum);
+		if (parsehost(p, hp)) {
+			sprintf(pvmtxt, "readhostfile() %s %d: errors\n", fn, lnum);
 			pvmlogerror(pvmtxt);
 			err++;
 			goto badline;
 		}
-		if (!ac)
-			goto badline;
-
-	/* add options to host descriptor */
-
-		while (--ac > 0) {
-			if (!strcmp(av[ac], "pw")) {
-#ifdef NOREXEC
-				sprintf(pvmtxt, "readhostfile() %s %d: can't do \"%s\"\n",
-						fn, lnum, av[ac]);
-				pvmlogerror(pvmtxt);
-				err++;
-#else
-				hp->hd_flag |= HF_ASKPW;
-#endif
-				continue;
-			}
-			if (!strcmp(av[ac], "ms")) {
-				hp->hd_flag |= HF_MANUAL;
-				continue;
-			}
-			if (!strncmp(av[ac], "lo=", 3)) {
-				if (hp->hd_login)
-					PVM_FREE(hp->hd_login);
-				hp->hd_login = STRALLOC(av[ac] + 3);
-				continue;
-			}
-			if (!strncmp(av[ac], "dx=", 3)) {
-				if (hp->hd_dpath)
-					PVM_FREE(hp->hd_dpath);
-				hp->hd_dpath = STRALLOC(av[ac] + 3);
-				continue;
-			}
-			if (!strncmp(av[ac], "ep=", 3)) {
-				if (hp->hd_epath)
-					PVM_FREE(hp->hd_epath);
-				hp->hd_epath = STRALLOC(av[ac] + 3);
-				continue;
-			}
-			if (!strncmp(av[ac], "sp=", 3)) {
-				hp->hd_speed = atoi(av[ac] + 3);
-				continue;
-			}
-			if (!strncmp(av[ac], "bx=", 3)) {
-				if (hp->hd_bpath)
-					PVM_FREE(hp->hd_bpath);
-				hp->hd_bpath = STRALLOC(av[ac] + 3);
-				continue;
-			}
-			sprintf(pvmtxt, "readhostfile() %s %d: unknown option \"%s\"\n",
-					fn, lnum, av[ac]);
-			pvmlogerror(pvmtxt);
-			err++;
-		}
 
 	/*
 	* if host == "*", change the default options
 	* instead of adding a new host to the list
 	*/
 
-		if (!strcmp(av[0], "*")) {
+		if (!strcmp(hp->hd_name, "*")) {
 			if (defaults)
 				hd_unref(defaults);
 			defaults = hp;
@@ -613,30 +730,12 @@ readhostfile(fn)
 
 	/* Set unspecified fields of hp to defaults */
 
-		if (defaults) {
-			hp->hd_flag |= defaults->hd_flag;
-
-			if (!hp->hd_login && defaults->hd_login)
-				hp->hd_login = STRALLOC(defaults->hd_login);
-
-			if (!hp->hd_dpath && defaults->hd_dpath)
-				hp->hd_dpath = STRALLOC(defaults->hd_dpath);
-
-			if (!hp->hd_epath && defaults->hd_epath)
-				hp->hd_epath = STRALLOC(defaults->hd_epath);
-
-			if (!hp->hd_bpath && defaults->hd_bpath)
-				hp->hd_bpath = STRALLOC(defaults->hd_bpath);
-		}
-
-		hp->hd_name = STRALLOC(av[0]);
-/*
-		hp->hd_arch = STRALLOC("");
-*/
+		if (defaults)
+			applydefaults(hp, defaults);
 
 	/* look up ip addr */
 
-		if (!(he = gethostbyname(hp->hd_name))) {
+		if (!(he = gethostbyname(hp->hd_aname ? hp->hd_aname : hp->hd_name))) {
 			sprintf(pvmtxt, "readhostfile() %s %d: %s: can't gethostbyname\n",
 				fn, lnum, hp->hd_name);
 			pvmlogerror(pvmtxt);
@@ -648,13 +747,14 @@ readhostfile(fn)
 
 	/* mark master host to not start */
 
-		for (i = num_addrs; i-- > 0; ) {
-			if (BCMP((char*)&my_in_addrs[i], (char*)&hp->hd_sad.sin_addr,
-					sizeof(struct in_addr)) == 0)
-				break;
-		}
-		if (i >= 0)
-			hp->hd_flag |= HF_NOSTART;
+		if (!(hp->hd_flag & HF_OVERLOAD))
+			for (i = num_addrs; i-- > 0; ) {
+				if (BCMP((char*)&my_in_addrs[i], (char*)&hp->hd_sad.sin_addr,
+						sizeof(struct in_addr)) == 0) {
+					hp->hd_flag |= HF_NOSTART;
+					break;
+				}
+			}
 
 		ht_insert(htp, hp);
 		hd_unref(hp);
@@ -670,7 +770,6 @@ badline:
 
 	if (defaults) {
 		defaults->hd_hostpart = 0;
-		defaults->hd_name = STRALLOC("*");
 		ht_insert(htp, defaults);
 	}
 	fclose(ff);
@@ -779,7 +878,7 @@ iflist(alp, np)
 	char **p;
 
 	if (iplist)
-		free(iplist);
+		PVM_FREE(iplist);
 	iplist = TALLOC(10, struct in_addr, "ifl");
 
 	if (gethostname(hn, sizeof(hn))) {
