@@ -1,11 +1,11 @@
 
 /*
- *           PVM 3.2:  Parallel Virtual Machine System 3.2
+ *         PVM version 3.3:  Parallel Virtual Machine System
  *               University of Tennessee, Knoxville TN.
  *           Oak Ridge National Laboratory, Oak Ridge TN.
  *                   Emory University, Atlanta GA.
  *      Authors:  A. L. Beguelin, J. J. Dongarra, G. A. Geist,
- *          R. J. Manchek, B. K. Moore, and V. S. Sunderam
+ *    W. C. Jiang, R. J. Manchek, B. K. Moore, and V. S. Sunderam
  *                   (C) 1992 All Rights Reserved
  *
  *                              NOTICE
@@ -22,8 +22,8 @@
  * purpose.  This software is provided ``as is'' without express or
  * implied warranty.
  *
- * PVM 3.2 was funded in part by the U.S. Department of Energy, the
- * National Science Foundation and the State of Tennessee.
+ * PVM version 3 was funded in part by the U.S. Department of Energy,
+ * the National Science Foundation and the State of Tennessee.
  */
 
 /*
@@ -35,6 +35,9 @@
  */
 
 
+#ifdef HASSTDLIB
+#include <stdlib.h>
+#endif
 #include <stdio.h>
 #include <math.h>
 #include <X11/Xlib.h>
@@ -302,7 +305,7 @@ start_workers()
 	int n;		/* actual number started */
 	int i;
 
-	pvm_config(&nserv, (int*)0, (struct hostinfo**)0);
+	pvm_config(&nserv, (int*)0, (struct pvmhostinfo**)0);
 	if (app_res.n > 0 && app_res.n <= nserv * 10)
 		nserv = app_res.n;
 /*
@@ -617,7 +620,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = quit_cb;
+	callback[0].callback = (XtCallbackProc)quit_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	pm = XCreatePixmapFromBitmapData(xDisp, xRootW,
@@ -631,7 +634,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = workers_cb;
+	callback[0].callback = (XtCallbackProc)workers_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -646,7 +649,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = redo_cb;
+	callback[0].callback = (XtCallbackProc)redo_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -661,7 +664,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = zout_cb;
+	callback[0].callback = (XtCallbackProc)zout_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -676,7 +679,7 @@ create_xep_widget()
 	XtSetArg(args[n], XtNright, XtChainLeft); n++;
 	XtSetArg(args[n], XtNtop, XtChainTop); n++;
 	XtSetArg(args[n], XtNbottom, XtChainTop); n++;
-	callback[0].callback = bars_cb;
+	callback[0].callback = (XtCallbackProc)bars_cb;
 	callback[0].closure = (caddr_t)0;
 	XtSetArg(args[n], XtNcallback, callback); n++;
 	XtSetArg(args[n], XtNfromHoriz, w); n++;
@@ -1401,6 +1404,7 @@ label_row(y1, y2, n)
 	int h = y2 - y1;
 	int y, x;
 	u_char *ba;
+	int c = isMono ? 128 : 240;
 
 	ba = imCan.cn_dat + y1 * wd;
 	for (y = h; y-- > 0; ) {
@@ -1412,7 +1416,7 @@ label_row(y1, y2, n)
 		ba += wd;
 		for (x = 0; x < 20; x++)
 			if (x & 3)
-				ba[x] = (n & (1 << x / 4)) ? 255 : 240;
+				ba[x] = (n & (1 << x / 4)) ? 255 : c;
 	}
 	repaint_region(&imCan, 0, y1, 20, y2 - 1);
 	refresh_region(&imCan, 0, y1, 20, y2 - 1);
